import axios from 'axios';
import { isNonEmptyString } from '@yugabytedb/ui-components';
import { REACT_APP_API_URL } from './helpers/config';
import { Anomaly, AnomalyDetectionStatus, GraphResponse, MetadataFields } from './helpers/dtos';

// define unique names to use them as query keys
export enum QUERY_KEY {
  fetchAnomalies = 'fetchAnomalies',
  fetchGraphs = 'fetchGraphs',
  fetchQueries = 'fetchQueries',
  fetchUniverseMetadataList = 'fetchUniverseMetadataList',
  deleteUniverseMetadata = 'deleteUniverseMetadata',
  updateUniverseMetadata = 'updateUniverseMetadata',
  fetchUniverseListDetails = 'fetchUniverseListDetails',
  fetchUniverseById = 'fetchUniverseById',
  fetchAnomalyDetectionStatus = 'fetchAnomalyDetectionStatus',
  scheduleAnomalyDetectionStatus = 'scheduleAnomalyDetectionStatus'
}

class ApiService {
  // Fetches list of anomalies
  fetchAnomalies = (
    universeUuid: string,
    startTime?: Date | null,
    endTime?: Date | null,
    apiUrl?: string
  ) => {
    console.log('fetchAnomalies startTime', startTime);
    console.log('fetchAnomalies endTime', endTime);
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/${universeUuid}/anomalies`;
    const params: any = {};
    if (startTime) {
      params.startTime = startTime;
    }
    if (endTime) {
      params.endTime = endTime;
    }
    return axios
      .get<Anomaly[]>(requestURL, {
        params: params
      })
      .then((res) => res.data);
  };

  fetchAnomaliesById = (universeUuid: string, anomalyUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/${universeUuid}/anomalies/${anomalyUuid}`;
    return axios.get<Anomaly>(requestURL).then((res) => res.data);
  };

  // Fetches graphs and supporting data for troubleshooting
  fetchGraphs = (universeUuid: String, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe/${universeUuid}/graphs`;
    return axios.post<GraphResponse[]>(requestUrl, data).then((resp) => resp.data);
  };

  fetchQueries = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/${universeUuid}/queries`;
    return axios.get<any>(requestURL).then((res) => res.data);
  };
  fetchCustomerMetadataList = (apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/customer/metadata`;
    return axios.get<MetadataFields[]>(requestURL).then((res) => res.data);
  };
  fetchCustomerMetadataById = (customerUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer/${customerUuid}/metadata`;
    return axios.get(requestUrl).then((res) => res.data);
  };
  updateCustomerMetadata = (customerUuid: string, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer/${customerUuid}/metadata`;
    return axios.put(requestUrl, data).then((resp) => resp.data);
  };
  deleteCustomerMetadataById = (customerUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/customer/${customerUuid}/metadata`;
    return axios.delete(requestUrl).then((resp) => resp.data);
  };
  fetchUniverseMetadataList = (apiUrl?: string, customerUuid?: string, universeUuid?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/metadata`;
    const params = {
      customer_uuid: customerUuid,
      universe_uuid: universeUuid
    };
    return axios
      .get<MetadataFields[]>(requestURL, { params: params })
      .then((res) => res.data);
  };
  updateUniverseMetadata = (universeUuid: string, data: any, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe/${universeUuid}/metadata`;
    return axios.put(requestUrl, data).then((resp) => resp.data);
  };
  deleteUniverseMetadata = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe/${universeUuid}/metadata`;
    return axios.delete(requestUrl).then((resp) => resp.data);
  };
  fetchUniverseListDetails = (apiUrl?: string, customerUuid?: string, universeUuid?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const params = {
      customer_uuid: customerUuid,
      universe_uuid: universeUuid
    };
    const requestUrl = `${baseUrl}/universe/details`;
    return axios.get(requestUrl, { params: params }).then((res) => res.data);
  };
  fetchUniverseById = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestUrl = `${baseUrl}/universe/${universeUuid}/details`;
    return axios.get(requestUrl).then((res) => res.data);
  };
  fetchAnomalyDetectionStatus = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/${universeUuid}/anomaly_detection_status`;
    return axios.get<AnomalyDetectionStatus>(requestURL).then((res) => res.data);
  };
  scheduleAnomalyDetectionStatus = (universeUuid: string, apiUrl?: string) => {
    const baseUrl = isNonEmptyString(apiUrl!) ? apiUrl : REACT_APP_API_URL;
    const requestURL = `${baseUrl}/universe/${universeUuid}/schedule_anomaly_detection`;
    return axios.post<AnomalyDetectionStatus>(requestURL, null).then((res) => res.data);
  };
}

export const TroubleshootAPI = new ApiService();
