import { useState, useEffect, ChangeEvent } from 'react';
import { useMutation, useQuery } from 'react-query';
import { useUpdateEffect } from 'react-use';
import { toast } from 'react-toastify';
import { Box, MenuItem, makeStyles } from '@material-ui/core';
import clsx from 'clsx';
import {
  YBSelect,
  YBDateTimePicker,
  YBTimeFormats,
  formatDatetime,
  getDiffHours,
  isEmptyString,
  isNonEmptyString,
  YBButton,
  isNonEmptyArray
} from '@yugabytedb/ui-components';
import { PrimaryDashboardTabs } from './PrimaryDashboardTabs';
import { QUERY_KEY, TroubleshootAPI } from '../api';
import { Anomaly, AnomalyCategory, AppName, PrimaryDashboardViewTabs } from '../helpers/dtos';
import { TIME_FILTER, anomalyFilterDurations } from '../helpers/constants';

interface PrimaryDashboardDataProps {
  anomalyData: Anomaly[] | null;
  appName: AppName;
  timezone?: string;
  universeUuid: string;
  apiUrl: string;
  onSelectedIssue?: (troubleshootUuid: string) => void;
  onFilterByDate: (startDate: any, endDate: any) => void;
}

const useStyles = makeStyles((theme) => ({
  troubleshootAdvisorBox: {
    marginRight: theme.spacing(1)
  },
  selectBox: {
    minWidth: '180px',
    maxWidth: '220px'
  },
  flexBoxRow: {
    display: 'flex',
    alignItems: 'center',
    flexDirection: 'row'
  },
  flexBoxRowReverse: {
    display: 'flex',
    alignItems: 'center',
    flexDirection: 'row-reverse'
  },
  dateBox: {
    alignItems: 'flex-start'
  },
  contentBox: {
    justifyContent: 'space-between'
  },
  menuItem: {
    display: 'block',
    padding: '15px 20px',
    height: '52px',
    whiteSpace: 'nowrap',
    fontSize: '14px'
  },
  overrideMuiInput: {
    '& .MuiInput-input': {
      fontWeight: 300,
      fontSize: '14px'
    },
    '& .MuiInput-root': {
      borderRadius: '8px 8px 8px 8px'
    },
    '& .MuiMenu-list': {
      maxHeight: '400px'
    }
  },
  regularText: {
    fontWeight: 300
  },
  datePicker: {
    marginRight: theme.spacing(2),
    marginTop: '-17px'
  },
  detectionButton: {
    marginRight: theme.spacing(2)
  },
  floatRight: {
    float: 'right'
  }
}));

const ANOMALY_CATEGORY_LIST = [
  AnomalyCategory.SQL,
  AnomalyCategory.NODE,
  AnomalyCategory.DB,
  AnomalyCategory.APP,
  AnomalyCategory.INFRA
];

export const PrimaryDashboardData = ({
  anomalyData,
  appName,
  universeUuid,
  timezone,
  apiUrl,
  onSelectedIssue,
  onFilterByDate
}: PrimaryDashboardDataProps) => {
  const classes = useStyles();

  const today = new Date();
  let todayDup = new Date();
  let previousDate = new Date();
  // The default window is 14 days
  previousDate.setDate(todayDup.getDate() - 14);

  const [currentTab, setCurrentTab] = useState<PrimaryDashboardViewTabs>(
    PrimaryDashboardViewTabs.ANOMALY
  );
  const [filteredAnomalyList, setFilteredAnomalyList] = useState<Anomaly[]>([]);
  const [anomalyDetectionStatusTime, setAnomalyDetectionStatusTime] = useState<string | null>(null);
  const [anomalyCategoryOptions, setAnomalyCategoryOptions] = useState<AnomalyCategory[]>();
  const [anomalyDuration, setAnomalyDuration] = useState<string>(anomalyFilterDurations[0].value);
  const [datetimeError, setDateTimeError] = useState<string>('');
  const [customStartDateTime, setCustomStartDateTime] = useState<Date>(previousDate);
  const [customEndDateTime, setCustomEndDateTime] = useState<Date>(today);
  const [anomalyDetectionStatus, setAnomalyDetectionStatus] = useState<string>('');

  useQuery(
    QUERY_KEY.fetchAnomalyDetectionStatus,
    () => TroubleshootAPI.fetchAnomalyDetectionStatus(universeUuid, apiUrl),
    {
      onSuccess: (data) => {
        const lastDetectionTime = data.detectionEndTime;
        if (!lastDetectionTime || isEmptyString(lastDetectionTime)) {
          setAnomalyDetectionStatusTime(null);
        } else {
          const date = new Date(lastDetectionTime);
          const localDate = formatDatetime(date, YBTimeFormats.YB_DATE_TIME_TIMESTAMP, timezone);
          setAnomalyDetectionStatusTime(localDate);
        }
        setAnomalyDetectionStatus(data.status);
      },
      onError: (error: any) => {
        console.error('Failed to fetch anomaly detection status', error);
      },
      refetchOnWindowFocus: true,
      refetchInterval: 30000
    }
  );

  const scheduleAnomalyDetectionStatus = useMutation(
    () => TroubleshootAPI.scheduleAnomalyDetectionStatus(universeUuid, apiUrl),
    {
      onSuccess: (data) => {
        const lastDetectionTime = data.detectionEndTime;
        if (!lastDetectionTime || isEmptyString(lastDetectionTime)) {
          setAnomalyDetectionStatusTime(null);
        } else {
          const date = new Date(lastDetectionTime);
          const localDate = formatDatetime(date, YBTimeFormats.YB_DATE_TIME_TIMESTAMP, timezone);
          setAnomalyDetectionStatusTime(localDate);
        }
        setAnomalyDetectionStatus(data.status);
      },
      onError: (e: any) => {
        const defaultMessage = 'Failed to schedule anomaly detection';
        toast.error(e?.response?.data?.error ?? defaultMessage);
      }
    }
  );

  // format date
  const currentDateTime = formatDatetime(
    customEndDateTime,
    YBTimeFormats.YB_DATE_TIME_TIMESTAMP,
    timezone
  );
  const previousDateTime = formatDatetime(
    customStartDateTime,
    YBTimeFormats.YB_DATE_TIME_TIMESTAMP,
    timezone
  );

  useEffect(() => {
    setFilteredAnomalyList(anomalyData!);
    isNonEmptyArray(anomalyData)
      ? setAnomalyCategoryOptions(anomalyData?.map((anomaly: Anomaly) => anomaly.category))
      : setAnomalyCategoryOptions([]);
  }, [anomalyData]);

  useUpdateEffect(() => {
    if (customStartDateTime > customEndDateTime) {
      setDateTimeError('Start Date is greater than end date');
    } else if (getDiffHours(customStartDateTime, customEndDateTime) < 24) {
      setDateTimeError('Time difference should be minimum of 24 hours');
    } else {
      // Make an API call with update timestamp
      setDateTimeError('');
      // Investigate if we can just do toISOString();
      const formattedStartDate = customStartDateTime?.toISOString();
      const formattedEndDate = customEndDateTime?.toISOString();
      onFilterByDate(formattedStartDate, formattedEndDate);
    }
  }, [customStartDateTime, customEndDateTime]);

  useUpdateEffect(() => {
    let formattedStartDate = previousDate?.toISOString();
    let formattedEndDate = today?.toISOString();
    let customDate = new Date();

    // When the window is 7 days
    if (anomalyDuration === '7d') {
      customDate.setDate(customDate.getDate() - 7);
      formattedStartDate = customDate?.toISOString();
      // When the window is 2 days
    } else if (anomalyDuration === '2d') {
      customDate.setDate(customDate.getDate() - 2);
      formattedStartDate = customDate?.toISOString();
      // When the window is 1 days
    } else if (anomalyDuration === '1d') {
      customDate.setDate(customDate.getDate() - 1);
      formattedStartDate = customDate?.toISOString();
    }
    onFilterByDate(formattedStartDate, formattedEndDate);
  }, [anomalyDuration]);

  const handleResolve = (id: string, isResolved: boolean) => {
    const anomalyListCopy = JSON.parse(JSON.stringify(filteredAnomalyList));
    const selectedAnomaly = anomalyListCopy?.find((anomaly: Anomaly) => anomaly.uuid === id);
    if (selectedAnomaly) {
      selectedAnomaly.isResolved = isResolved;
    }
    setFilteredAnomalyList(anomalyListCopy);
  };

  const onSelectCategories = (category: AnomalyCategory | string) => {
    const anomalyListCopy = JSON.parse(JSON.stringify(anomalyData));
    const filteredAnomalies =
      category !== 'ALL'
        ? anomalyListCopy?.filter((anomaly: Anomaly) => anomaly.category === category)
        : anomalyData;
    setFilteredAnomalyList(filteredAnomalies);
  };

  const onSelectedFilterDuration = (filterDuration: string) => {
    setAnomalyDuration(filterDuration);
  };

  const onStartDateChange = (e: ChangeEvent<HTMLInputElement>) => {
    setCustomStartDateTime(new Date(e.target.value));
  };

  const onEndDateChange = (e: ChangeEvent<HTMLInputElement>) => {
    setCustomEndDateTime(new Date(e.target.value));
  };

  const onTabChange = (tab: PrimaryDashboardViewTabs) => {
    setCurrentTab(tab);
  };

  return (
    <Box>
      <Box
        className={clsx(
          currentTab === PrimaryDashboardViewTabs.QUERY
            ? classes.flexBoxRowReverse
            : classes.flexBoxRow,
          classes.contentBox
        )}
      >
        <Box>
          <YBSelect
            className={clsx(classes.selectBox, classes.overrideMuiInput)}
            hidden={currentTab === PrimaryDashboardViewTabs.QUERY}
          >
            <MenuItem
              className={classes.menuItem}
              onClick={() => {
                onSelectCategories('ALL');
              }}
            >
              {'ALL'}
            </MenuItem>
            {ANOMALY_CATEGORY_LIST?.map((category: AnomalyCategory, idx: number) => {
              return (
                <MenuItem
                  className={classes.menuItem}
                  key={idx}
                  disabled={!anomalyCategoryOptions?.includes(category)}
                  value={category}
                  onClick={() => {
                    onSelectCategories(category);
                  }}
                >
                  {`${category} ISSUE`}
                </MenuItem>
              );
            })}
          </YBSelect>
        </Box>

        <Box className={classes.flexBoxRow}>
          {isNonEmptyString(anomalyDetectionStatus) && (
            <Box mr={2} hidden={currentTab === PrimaryDashboardViewTabs.QUERY}>
              <span style={{ alignSelf: 'center' }}>
                {`Detection Status: ${anomalyDetectionStatus}`}
              </span>
            </Box>
          )}
          {isNonEmptyString(anomalyDetectionStatusTime!) && (
            <Box mr={2} hidden={currentTab === PrimaryDashboardViewTabs.QUERY}>
              <span style={{ alignSelf: 'center' }}>
                {`Last detection time: ${anomalyDetectionStatusTime}`}
              </span>
            </Box>
          )}

          <YBButton
            variant="primary"
            size="large"
            onClick={() => {
              scheduleAnomalyDetectionStatus.mutate();
            }}
            hidden={currentTab === PrimaryDashboardViewTabs.QUERY}
            className={classes.detectionButton}
            disabled={
              anomalyDetectionStatus === 'SCHEDULED' || anomalyDetectionStatus === 'IN_PROGRESS'
            }
          >
            {'Run Detection'}
          </YBButton>

          <YBSelect
            className={clsx(classes.selectBox, classes.overrideMuiInput)}
            data-testid={`PrimaryDashboard-TimeFilterSelect`}
            value={anomalyDuration}
          >
            {anomalyFilterDurations.map((duration: any) => (
              <MenuItem
                key={`duration-${duration.label}`}
                value={duration.value}
                onClick={(e: any) => {
                  onSelectedFilterDuration(duration.value);
                }}
                className={clsx(classes.menuItem, classes.regularText)}
              >
                {duration.label}
              </MenuItem>
            ))}
          </YBSelect>
          {anomalyDuration === TIME_FILTER.SMALL_CUSTOM && (
            <>
              <Box className={classes.datePicker}>
                <YBDateTimePicker
                  dateTimeLabel={'Start Date'}
                  defaultDateTimeValue={previousDateTime}
                  onChange={onStartDateChange}
                />
              </Box>
              <Box className={classes.datePicker}>
                <YBDateTimePicker
                  dateTimeLabel={'End Date'}
                  defaultDateTimeValue={currentDateTime}
                  onChange={onEndDateChange}
                  errorMsg={datetimeError}
                />
              </Box>
            </>
          )}
        </Box>
      </Box>

      <Box className={classes.troubleshootAdvisorBox} mt={3}>
        <PrimaryDashboardTabs
          filteredAnomalyList={filteredAnomalyList}
          handleResolve={handleResolve}
          universeUuid={universeUuid}
          appName={appName}
          timezone={timezone}
          onSelectedIssue={onSelectedIssue}
          currentTab={currentTab}
          onTabChange={onTabChange}
        />
      </Box>
    </Box>
  );
};
