import { useState } from 'react';
import { Box, Typography } from '@material-ui/core';
import clsx from 'clsx';
import {
  isEmptyArray,
  isNonEmptyArray,
  isNonEmptyObject,
  isNonEmptyString,
  YBPanelItem
} from '@yugabytedb/ui-components';
import { SecondaryDashboard } from './SecondaryDashboard';
import { AppName, GraphResponse, GraphType, MetricMeasure } from '../helpers/dtos';
import { getOperations } from '../helpers/utils';
import { useHelperStyles } from './styles';

import TraingleDownIcon from '../assets/traingle-down.svg';
import TraingleUpIcon from '../assets/traingle-up.svg';

interface RCAViewProps {
  metricMeasure: string;
  anomalyData: any;
  appName: AppName;
  timezone?: string;
  universeQueryData: any;
  graphQueryParams: any;
  recommendationMetrics: any;
  graphData: GraphResponse[] | null;
  onSelectAshLabel: (groupByLabel: string, metricData: any, isAdvancedView?: boolean) => void;
}

export const RCAView = ({
  metricMeasure,
  anomalyData,
  appName,
  timezone,
  universeQueryData,
  graphQueryParams,
  recommendationMetrics,
  graphData,
  onSelectAshLabel
}: RCAViewProps) => {
  const classes = useHelperStyles();
  const [openTiles, setOpenTiles] = useState<string[]>([]);
  const handleOpenBox = (metricName: string) => {
    let openTilesCopy = [...openTiles];

    if (!openTilesCopy.includes(metricName)) {
      openTilesCopy.push(metricName);
      setOpenTiles(openTilesCopy);
    } else if (openTilesCopy.includes(metricName)) {
      const openTileIndex = openTilesCopy.indexOf(metricName);
      if (openTileIndex >= 0) {
        openTilesCopy.splice(openTileIndex, 1);
      }
      setOpenTiles(openTilesCopy);
    }
  };

  const renderSupportingGraphs = (
    graphResponseData: any,
    uniqueOperations: any,
    groupByOperations: any,
    graphType: GraphType
  ) => {
    return (
      <Box mt={3} mr={8}>
        <SecondaryDashboard
          metricData={graphResponseData}
          metricKey={`${graphResponseData.name}-${Math.random() * 1000}`}
          containerWidth={null}
          prometheusQueryEnabled={true}
          metricMeasure={metricMeasure}
          operations={uniqueOperations}
          groupByOperations={groupByOperations}
          shouldAbbreviateTraceName={true}
          isMetricLoading={false}
          anomalyData={anomalyData}
          appName={appName}
          timezone={timezone}
          graphType={graphType}
          queryData={universeQueryData}
          onSelectAshLabel={onSelectAshLabel}
        />
      </Box>
    );
  };

  return (
    <Box mt={4}>
      <YBPanelItem
        body={
          <>
            {isNonEmptyArray(graphData) &&
              isNonEmptyArray(anomalyData?.mainGraphs) &&
              anomalyData?.mainGraphs.map((graph: any, graphIdx: number) => {
                const metricData = graphData?.find((data: any) => data.name === graph.name);
                // we need to show group by operations only if anomaly response
                // object for a graph contains true for groupByControl
                const shouldShowGroupbyOperations = graph.groupByControl;
                // ASH can have buttons (operations) in OVERALL, but it should not have any controls if it is the MAIN graph
                const operations =
                  metricMeasure === MetricMeasure.OVERALL
                    ? { uniqueOperations: [], groupByOperations: [] }
                    : getOperations(
                        metricMeasure,
                        metricData,
                        GraphType.MAIN,
                        shouldShowGroupbyOperations,
                        graphQueryParams
                      );

                return (
                  <Box className={classes.secondaryDashboard}>
                    {graphIdx === 0 && (
                      <Box mt={1} ml={0.5}>
                        <span className={clsx(classes.largeBold)}>
                          {`${anomalyData?.category} Issue: ${anomalyData?.summary} `}
                        </span>
                      </Box>
                    )}
                    {renderSupportingGraphs(
                      metricData,
                      operations.uniqueOperations,
                      operations.groupByOperations,
                      GraphType.MAIN
                    )}
                  </Box>
                );
              })}
            {isNonEmptyArray(graphData) &&
              // Display metrics in the same order based on request params tp graph
              // This will help us to group metrics together based on RCA Guidelines
              recommendationMetrics?.map((reason: any, reasonIdx: number) => {
                let renderItems: any = [];
                if (isEmptyArray(reason?.name)) {
                  return (
                    <Box
                      onClick={() => handleOpenBox(reason.cause)}
                      className={classes.secondaryDashboard}
                      key={reason.cause}
                    >
                      <Box>
                        <span className={classes.smallBold}>{reason.cause}</span>
                      </Box>
                      <Box mt={1}>
                        <span className={classes.mediumNormal}>{reason.description}</span>
                      </Box>
                      {openTiles.includes(reason.cause) ? (
                        <img src={TraingleDownIcon} alt="expand" className={classes.arrowIcon} />
                      ) : (
                        <img src={TraingleUpIcon} alt="shrink" className={classes.arrowIcon} />
                      )}
                      {openTiles.includes(reason.cause) && (
                        <Box mt={3}>
                          <span className={classes.smallNormal}>
                            {'THERE ARE NO SUPPORTING METRICS'}
                          </span>
                        </Box>
                      )}
                    </Box>
                  );
                } else {
                  return reason?.name?.map((metricName: string, idx: number) => {
                    const numReasons = reason.name.length - 1;
                    const isOnlyReason = reason.name.length === 1;
                    const metricData: any = graphData?.find((data: any, graphIdx: number) => {
                      // anomalyData?.mainGraphs?.length - 1 condition decides the index mapping of graphs for
                      // supporting graphs based on number of main graphs
                      const supportingGraphsIndexCondition =
                        anomalyData.mainGraphs.length === 1
                          ? graphIdx === reason.index[idx]
                          : graphIdx - anomalyData.mainGraphs.length + 1 === reason.index[idx];
                      return (
                        graphIdx > 0 && data.name === metricName && supportingGraphsIndexCondition
                      );
                    });
                    const shouldShowGroupbyOperations = reason.groupByControl?.[0];
                    if (isNonEmptyString(metricData?.errorMessage)) {
                      return <Box>{}</Box>;
                    } else {
                      const operations = getOperations(
                        metricMeasure,
                        metricData,
                        GraphType.SUPPORTING,
                        shouldShowGroupbyOperations,
                        graphQueryParams
                      );

                      return (
                        <>
                          {idx === 0 && reasonIdx === 0 && (
                            <Box mt={2} mb={2}>
                              <Typography variant="h4">{'Possible Causes'}</Typography>
                            </Box>
                          )}
                          {isNonEmptyObject(metricData) && (
                            <>
                              <Box hidden={true}>
                                {renderItems.push(
                                  renderSupportingGraphs(
                                    metricData,
                                    operations.uniqueOperations,
                                    operations.groupByOperations,
                                    isOnlyReason ? GraphType.MAIN : GraphType.SUPPORTING
                                  )
                                )}
                              </Box>
                              {idx === numReasons && (
                                <Box
                                  onClick={() => handleOpenBox(metricData.name)}
                                  className={classes.secondaryDashboard}
                                  key={metricData.name}
                                >
                                  <Box>
                                    <span className={classes.smallBold}>{reason.cause}</span>
                                  </Box>
                                  <Box mt={1}>
                                    <span className={classes.mediumNormal}>
                                      {reason.description}
                                    </span>
                                  </Box>
                                  {openTiles.includes(metricData.name) ? (
                                    <img
                                      src={TraingleDownIcon}
                                      alt="expand"
                                      className={classes.arrowIcon}
                                    />
                                  ) : (
                                    <img
                                      src={TraingleUpIcon}
                                      alt="shrink"
                                      className={classes.arrowIcon}
                                    />
                                  )}
                                  {openTiles.includes(metricData.name) && (
                                    <Box mt={3}>
                                      <span className={classes.smallNormal}>
                                        {'SUPPORTING METRICS'}
                                      </span>
                                    </Box>
                                  )}
                                  {openTiles.includes(metricData.name) && (
                                    <Box className={clsx(classes.metricGroupItems)}>
                                      {renderItems?.map((item: any) => {
                                        return item;
                                      })}
                                    </Box>
                                  )}
                                </Box>
                              )}
                            </>
                          )}
                        </>
                      );
                    }
                  });
                }
              })}
          </>
        }
      />
    </Box>
  );
};
