import { useEffect, useRef, useState } from 'react';
import { usePrevious } from 'react-use';
import clsx from 'clsx';
import _ from 'lodash';
import { Tooltip, Box, makeStyles, MenuItem } from '@material-ui/core';
import moment from 'moment-timezone';
import {
  YBSelect,
  isNonEmptyArray,
  isNonEmptyObject,
  removeNullProperties,
  isYAxisGreaterThanThousand,
  divideYAxisByThousand,
  timeFormatXAxis,
  isNonEmptyString
} from '@yugabytedb/ui-components';
import { Anomaly, AppName, GraphLabel, GraphType, MetricMeasure } from '../helpers/dtos';
import { ASH, METRIC_FONT, MetricConsts } from '../helpers/constants';

import PrometheusIcon from '../assets/prometheus-icon.svg';

const Plotly = require('plotly.js/lib/index-basic.js');

interface SecondaryDashboardProps {
  anomalyData: Anomaly | null;
  metricData: any;
  appName: AppName;
  metricKey: string;
  containerWidth: number | null;
  prometheusQueryEnabled: boolean;
  metricMeasure: string;
  operations: string[];
  groupByOperations: string[];
  height?: number;
  width?: number;
  shouldAbbreviateTraceName: boolean;
  isMetricLoading: boolean;
  timezone?: string;
  className?: any;
  graphType: GraphType;
  queryData?: any;
  isAdvancedView?: boolean;
  onSelectAshLabel: (groupBy: string, metricData: any, isAdvancedView?: boolean) => void;
}

const COLORS = [
  '#1f77b4', // muted blue
  '#ff7f0e', // safety orange
  '#2ca02c', // cooked asparagus green
  '#d62728', // brick red
  '#9467bd', // muted purple
  '#8c564b', // chestnut brown
  '#e377c2', // raspberry yogurt pink
  '#7f7f7f', // middle gray
  '#bcbd22', // curry yellow-green
  '#17becf' // blue-teal
];

const WIDTH_OFFSET = 23;
const CONTAINER_PADDING = 60;
const MAX_GRAPH_WIDTH_PX = 1600;
const GRAPH_GUTTER_WIDTH_PX = 0;
const MAX_NAME_LENGTH = 25;

const DEFAULT_HEIGHT = 400;
const DEFAULT_CONTAINER_WIDTH = 1600;

const DEFAULT_CONTAINER_WIDTH_YBM = 1400;
const MAX_GRAPH_WIDTH_PX_YBM = 1400;

const WIDTH_OFFSET_OTHER = 23;

const MAX_GRAPH_WIDTH_PX_OTHER = 600;
const DEFAULT_CONTAINER_WIDTH_OTHER = 1200;

const useStyles = makeStyles(() => ({
  outlierOnlyButton: { borderRadius: '8px' },
  metricPanel: {
    padding: '1px',
    backgroundColor: '#dedee0',
    margin: '5px 7.5px',
    position: 'relative',
    overflow: 'hidden',
    textOverflow: 'ellipsis',
    whiteSpace: 'nowrap',
    maxWidth: 'fit-content'
  },
  overideMuiSelect: {
    '& .MuiInput-input': {
      color: '#2b59c3 !important',
      background: 'linear-gradient(0deg, rgba(43, 89, 195, 0.1), rgba(43, 89, 195, 0.1)), #FFFFFF'
    },
    '& .MuiInput-root': {
      color: '#2b59c3 !important',

      background: 'linear-gradient(0deg, rgba(43, 89, 195, 0.1), rgba(43, 89, 195, 0.1)), #FFFFFF'
    }
  },
  overrideDefaultMuiSelect: {
    maxHeight: '200px',
    overflowY: 'scroll',
    '& .MuiInput-root': {
      height: '32px'
    }
  },
  overrideDefaultMuiFormControl: {
    flexDirection: 'row',
    '& .MuiFormControl-root': {
      flexDirection: 'row'
    }
  },
  menuItem: {
    fontWeight: 300,
    fontSize: '14px',
    padding: '15px 20px',
    height: '52px'
  },
  outlierButtonContainer: {
    position: 'absolute',
    top: '20px',
    right: '50px',
    display: 'flex',
    flex: 1,
    flexDirection: 'row'
  },
  prometheusIcon: {
    position: 'absolute',
    top: '15px',
    right: '15px',
    marginLeft: '17px',
    marginTop: '5px'
  }
}));

export const SecondaryDashboard = ({
  metricData,
  appName,
  metricKey,
  containerWidth,
  prometheusQueryEnabled,
  metricMeasure,
  operations,
  groupByOperations,
  anomalyData,
  height,
  width,
  shouldAbbreviateTraceName,
  isMetricLoading,
  className,
  timezone,
  graphType,
  queryData,
  isAdvancedView,
  onSelectAshLabel
}: SecondaryDashboardProps) => {
  const classes = useStyles();
  let metricOperationsDisplayed = operations;

  const anomalyStartTime = anomalyData?.startTime;
  const anomalyEndTime = anomalyData?.endTime;
  const utcStartTimeStamp = moment.utc(anomalyStartTime).valueOf();
  const utcEndTimeStamp = anomalyEndTime ? moment.utc(anomalyEndTime).valueOf() : null;

  // State variables
  const [outlierButtonsWidth, setOutlierButtonsWidth] = useState<number | null>(null);
  const [focusedButton, setFocusedButton] = useState<any>(null);
  const [selectedGroupByOperation, setSelectedGroupByOperation] = useState<any>(
    groupByOperations?.[0]
  );
  const [selectedMetricOperation, setSelectedMetricOperation] = useState<any>(
    metricOperationsDisplayed?.[0]
  );

  // Ref and Previous variavles
  const outlierButtonsRef: any = useRef(null);
  const previousMetricData = usePrevious(metricData);

  const getGraphWidth = (containerWidth: number) => {
    let maxGraphWidth =
      appName === AppName.YBA
        ? graphType === GraphType.MAIN
          ? MAX_GRAPH_WIDTH_PX
          : MAX_GRAPH_WIDTH_PX_OTHER
        : MAX_GRAPH_WIDTH_PX_YBM;
    if (appName === AppName.YBM) {
      maxGraphWidth =
        graphType === GraphType.MAIN ? MAX_GRAPH_WIDTH_PX_YBM : MAX_GRAPH_WIDTH_PX_OTHER;
    }
    const width =
      containerWidth -
      CONTAINER_PADDING -
      (graphType === GraphType.MAIN ? WIDTH_OFFSET : WIDTH_OFFSET_OTHER);
    const columnCount = Math.ceil(width / maxGraphWidth);
    return Math.floor(width / columnCount) - GRAPH_GUTTER_WIDTH_PX;
  };

  const plotGraph = (metricOperation: string | null) => {
    const metric = _.cloneDeep(metricData);
    if (isNonEmptyObject(metric)) {
      const layoutHeight = height ?? DEFAULT_HEIGHT;

      let graphWidth =
        appName === AppName.YBA ? DEFAULT_CONTAINER_WIDTH : DEFAULT_CONTAINER_WIDTH_YBM;
      if (appName === AppName.YBA) {
        graphWidth =
          graphType === GraphType.MAIN ? DEFAULT_CONTAINER_WIDTH : DEFAULT_CONTAINER_WIDTH_OTHER;
      } else if (appName === AppName.YBM) {
        graphWidth =
          graphType === GraphType.MAIN
            ? DEFAULT_CONTAINER_WIDTH_YBM
            : DEFAULT_CONTAINER_WIDTH_OTHER;
      }
      const layoutWidth = width ?? getGraphWidth(graphWidth);

      const matchingMetricOperation = metricOperation
        ? metricOperation
        : focusedButton ?? operations[0];
      if (
        (operations.length || metricOperation) &&
        (metricMeasure === MetricMeasure.OUTLIER || metricMeasure === MetricMeasure.OUTLIER_TABLES)
      ) {
        metric.data = metric.data.filter(
          (dataItem: any) => dataItem.name === matchingMetricOperation
        );
      }

      metric.data.forEach((dataItem: any, i: number) => {
        // COLORS array has 10 colors, if we have more than 10 traces, we will start reusing colors
        dataItem.marker = {
          color: COLORS[i % COLORS.length]
        };
        if (dataItem['instanceName'] && dataItem['name'] !== dataItem['instanceName']) {
          dataItem['fullname'] = dataItem['name'] + ' (' + dataItem['instanceName'] + ')';
          if (metricMeasure === MetricMeasure.OUTLIER) {
            dataItem['fullname'] = dataItem['instanceName'];
          }
          // If the metrics API returns data without instanceName field in case of
          // outliers, it belongs to cluster average data
        } else if (metricMeasure === MetricMeasure.OUTLIER && !dataItem['instanceName']) {
          dataItem['fullname'] = MetricConsts.NODE_AVERAGE;
        } else if (metricMeasure === MetricMeasure.OUTLIER_TABLES) {
          dataItem['fullname'] = dataItem['tableName'];
        } else if (
          metric?.layout?.type === ASH &&
          metricMeasure === MetricMeasure.OVERALL &&
          metric?.layout?.metadata?.currentGroupBy?.[0] === GraphLabel.QUERY_ID
        ) {
          const queryObject = queryData?.find((data: any) => {
            return data.id.queryId === dataItem['name'];
          });
          dataItem['fullname'] = queryObject?.query;
        } else {
          dataItem['fullname'] = dataItem['name'];
        }

        // To avoid the legend overlapping plot, we allow the option to abbreviate trace names if:
        // - received truthy shouldAbbreviateTraceName AND
        // - trace name longer than the max name legnth
        const shouldAbbreviate =
          shouldAbbreviateTraceName && dataItem['name'].length > MAX_NAME_LENGTH;
        if (
          metric?.layout?.type === ASH &&
          metricMeasure === MetricMeasure.OVERALL &&
          metric?.layout?.metadata?.currentGroupBy?.[0] === GraphLabel.QUERY_ID
        ) {
          dataItem['name'] = dataItem['fullname']?.substring(0, MAX_NAME_LENGTH) + '...';
        } else if (shouldAbbreviate && !(metricMeasure === MetricMeasure.OUTLIER)) {
          dataItem['name'] = dataItem['name'].substring(0, MAX_NAME_LENGTH) + '...';
          // Legend name from outlier should be based on instance name in case of outliers
        } else if (metricMeasure === MetricMeasure.OUTLIER) {
          dataItem['name'] = dataItem['instanceName']
            ? dataItem['instanceName'] + ' (' + dataItem['name'] + ')'
            : MetricConsts.NODE_AVERAGE;
        } else if (metricMeasure === MetricMeasure.OUTLIER_TABLES) {
          dataItem['name'] = dataItem['namespaceName']
            ? `${dataItem['namespaceName']}.${dataItem['tableName']}`
            : dataItem['tableName'];
        }
        // Only show upto first 8 traces in the legend
        if (i >= 8) {
          dataItem['showlegend'] = false;
        }
      });
      // Remove Null Properties from the layout
      removeNullProperties(metric.layout);
      // Detect if unit is µs and Y axis value is > 1000.
      // if so divide all Y axis values by 1000 and replace unit to ms.
      if (
        isNonEmptyObject(metric.layout.yaxis) &&
        metric.layout.yaxis.ticksuffix === '&nbsp;µs' &&
        isNonEmptyArray(metric.data)
      ) {
        if (isYAxisGreaterThanThousand(metric.data)) {
          metric.data = divideYAxisByThousand(metric.data);
          metric.layout.yaxis.ticksuffix = '&nbsp;ms';
        }
      }

      metric.data = timeFormatXAxis(metric.data, timezone);

      metric.layout.xaxis.hoverformat = timezone
        ? '%H:%M:%S, %b %d, %Y ' + moment.tz(timezone).format('[UTC]ZZ')
        : '%H:%M:%S, %b %d, %Y ' + moment().format('[UTC]ZZ');

      let max = 0;
      const isASHWithOperations =
        metric?.layout?.type === ASH &&
        !(
          metric?.layout?.metadata?.currentGroupBy?.[0] === GraphLabel.QUERY_ID ||
          metric?.layout?.metadata?.currentGroupBy?.[0] === GraphLabel.CLIENT_NODE_IP
        ) &&
        metricMeasure === MetricMeasure.OVERALL;

      metric.data.forEach((data: any) => {
        if (isASHWithOperations) {
          data.fill = 'tonexty';
          data.stackgroup = 'one';
          data.type = 'scatter';
        }
        if (metricMeasure === MetricMeasure.OUTLIER_TABLES && data?.namespaceName) {
          data.hovertemplate =
            '%{data.namespaceName}.%{data.fullname}: %{y} at %{x} <extra></extra>';
        } else {
          data.hovertemplate = '%{data.fullname}: %{y} at %{x} <extra></extra>';
        }

        if (data.y && isASHWithOperations) {
          data.y.forEach((y: any) => {
            y = parseFloat(y) * 2.5;
            if (y > max) max = y;
          });
        } else {
          data.y.forEach((y: any) => {
            y = parseFloat(y) * 1.25;
            if (y > max) max = y;
          });
        }
      });

      // TOOD: Get screen width using window.screen.availWidth
      if (max === 0) max = 1.01;
      metric.layout.autosize = false;
      metric.layout.width = layoutWidth;
      metric.layout.height = layoutHeight;
      metric.layout.showlegend = true;
      metric.layout.shapes = [
        {
          type: 'line',
          // x-reference is assigned to the x-values
          xref: 'x',
          // y-reference is assigned to the plot paper [0,1]
          yref: 'paper',
          x0: utcStartTimeStamp,
          y0: 0,
          x1: utcStartTimeStamp,
          y1: 1,
          fillcolor: '#d3d3d3',
          opacity: 0.2,
          line: {
            color: 'red',
            width: utcEndTimeStamp ? 2.5 : 1.5,
            dash: 'dot'
          }
        },
        {
          type: 'line',
          xref: 'x',
          yref: 'paper',
          x0: utcEndTimeStamp ?? utcStartTimeStamp,
          y0: 0,
          x1: utcEndTimeStamp ?? utcStartTimeStamp,
          y1: 1,
          fillcolor: '#d3d3d3',
          opacity: 0.2,
          line: {
            color: 'red',
            width: utcEndTimeStamp ? 2.5 : 1.5,
            dash: 'dot'
          }
        }
      ];

      metric.layout.title = {
        text: metric.layout?.title,
        x: 0.05,
        y: 2.2,
        xref: 'container',
        yref: 'container',
        font: {
          family: 'Inter',
          size: 15,
          color: '#0B1117'
        }
      };
      metric.layout.hovermode = 'closest';
      metric.layout.margin = {
        l: 45,
        r: 25,
        b: 0,
        t: 70,
        pad: 4
      };
      if (
        isNonEmptyObject(metric.layout.yaxis) &&
        isNonEmptyString(metric.layout.yaxis.ticksuffix)
      ) {
        metric.layout.margin.l = 70;
        metric.layout.yaxis.range = [0, max];
      } else {
        metric.layout.yaxis = { range: [0, max] };
      }
      metric.layout.font = {
        family: METRIC_FONT
      };

      metric.layout.legend = {
        orientation: 'h',
        y: -0.3
      };

      // Handle the case when the metric data is empty, we would show
      // graph with No Data annotation.
      if (!isNonEmptyArray(metric.data)) {
        metric.layout['annotations'] = [
          {
            visible: true,
            align: 'center',
            text: isMetricLoading ? 'Loading metric...' : 'No Data',
            showarrow: false,
            x: 1,
            y: 1
          }
        ];
        metric.layout.margin.b = 105;
        metric.layout.xaxis = { range: [0, 2] };
        metric.layout.yaxis = { range: [0, 2] };
      }
      Plotly.newPlot(metricKey, metric.data, metric.layout, {
        displayModeBar: false,
        responsive: true
      });
    }
  };

  const loadDataByGroupByOperation = (groupByOperation: string, metricData: any) => {
    // Show groupBy operations even in OVERALL view when in ADVANCED Query view
    metricData?.layout?.type === ASH &&
      (metricMeasure === MetricMeasure.OUTLIER ||
        metricMeasure === MetricMeasure.OUTLIER_TABLES ||
        isAdvancedView) &&
      onSelectAshLabel(groupByOperation, metricData, !!isAdvancedView);
  };

  const loadDataByMetricOperation = (metricOperation: string, metricData: any) => {
    setFocusedButton(metricOperation);
    metricData?.layout?.type === ASH &&
    metricMeasure === MetricMeasure.OVERALL &&
    isNonEmptyArray(operations)
      ? onSelectAshLabel(metricOperation, metricData, !!isAdvancedView)
      : plotGraph(metricOperation);
  };

  // Plot graph during mount of the component with metric data
  useEffect(() => {
    plotGraph(null);
    const outlierButtonsWidth: any = outlierButtonsRef.current?.offsetWidth;
    setOutlierButtonsWidth(outlierButtonsWidth);
  }, []);

  useEffect(() => {
    if (containerWidth || width) {
      Plotly.relayout(metricKey, {
        width: width ?? getGraphWidth(containerWidth!)
      });
    }
  }, [containerWidth, width, metricData, operations]);

  useEffect(() => {
    if (previousMetricData && metricData && !_.isEqual(previousMetricData, metricData)) {
      if (!outlierButtonsWidth) {
        setOutlierButtonsWidth(outlierButtonsRef.current?.offsetWidth);
      }

      // Re-plot graph
      plotGraph(null);
    }
  }, [metricData]);

  // Update metric operation when groupByOperation is changed
  useEffect(() => {
    if (isNonEmptyArray(operations)) {
      setSelectedMetricOperation(operations?.[0]);
    }
    plotGraph(operations?.[0]);
  }, [groupByOperations, metricMeasure]);

  // Get prometheus link for URL
  const getMetricsUrl = (internalUrl: string, metricsLinkUseBrowserFqdn: boolean) => {
    if (!metricsLinkUseBrowserFqdn) {
      return internalUrl;
    }
    const url = new URL(internalUrl);
    url.hostname = window.location.hostname;
    return url.href;
  };

  const inFocusButton = focusedButton ? focusedButton : operations?.[0];

  const tooltip = (
    <Tooltip
      title={'Metric graph in Prometheus'}
      arrow
      placement="top"
      className={classes.prometheusIcon}
    >
      <img
        alt="Metric graph in Prometheus"
        src={PrometheusIcon}
        width="25"
        onClick={() => {
          const prometheusUrl = getMetricsUrl(
            metricData.directURLs[0],
            metricData.metricsLinkUseBrowserFqdn
          );
          window.open(prometheusUrl);
        }}
      />
    </Tooltip>
  );

  return (
    <Box
      id={metricKey}
      className={clsx(className, classes.metricPanel)}
      onClick={(e) => {
        e.stopPropagation();
      }}
    >
      <span ref={outlierButtonsRef} className={classes.outlierButtonContainer}>
        {metricData?.layout?.type === ASH &&
          (metricMeasure === MetricMeasure.OUTLIER ||
            metricMeasure === MetricMeasure.OUTLIER_TABLES ||
            isAdvancedView) &&
          isNonEmptyArray(groupByOperations) && (
            <Box ml={2} mr={2}>
              <YBSelect
                value={selectedGroupByOperation}
                className={clsx(
                  classes.outlierOnlyButton,
                  classes.overrideDefaultMuiSelect,
                  classes.overrideDefaultMuiFormControl
                )}
              >
                {groupByOperations?.map((groupByOperation: string, operationIdx: number) => {
                  return (
                    <MenuItem
                      key={operationIdx}
                      value={groupByOperation}
                      onClick={(event) => {
                        loadDataByGroupByOperation(groupByOperation, metricData);
                        setSelectedGroupByOperation(groupByOperation);
                        event.stopPropagation();
                      }}
                    >
                      {groupByOperation}
                    </MenuItem>
                  );
                })}
              </YBSelect>
            </Box>
          )}

        {isNonEmptyArray(metricOperationsDisplayed) && (
          <YBSelect
            value={selectedMetricOperation}
            className={clsx(
              classes.outlierOnlyButton,
              classes.overrideDefaultMuiSelect,
              classes.overrideDefaultMuiFormControl
            )}
          >
            {metricOperationsDisplayed?.map((operation: string, idx: number) => {
              return (
                <MenuItem
                  className={clsx(
                    operation === inFocusButton && classes.overideMuiSelect,
                    classes.menuItem
                  )}
                  key={`${operation}-${idx}`}
                  value={operation}
                  onClick={(event) => {
                    // setActive(true);
                    setSelectedMetricOperation(operation);
                    loadDataByMetricOperation(operation, metricData);
                    event.stopPropagation();
                  }}
                >
                  {operation}
                </MenuItem>
              );
            })}
          </YBSelect>
        )}
      </span>
      {prometheusQueryEnabled && isNonEmptyArray(metricData?.directURLs) ? <>{tooltip}</> : null}
      <div />
    </Box>
  );
};
