import { useEffect, useState, ChangeEvent } from 'react';
import { usePrevious } from 'react-use';
import { useMutation } from 'react-query';
import { useUpdateEffect } from 'react-use';
import { Box } from '@material-ui/core';
import _ from 'lodash';
import clsx from 'clsx';
import { isNonEmptyString } from '@yugabytedb/ui-components';
import { RCAView } from './RCAView';
import { QueryDataView } from './QueryDataView';
import { SecondaryDashboardHeader } from './SecondaryDashboardHeader';
import { TroubleshootAPI } from '../api';
import {
  Anomaly,
  AnomalyCategory,
  AppName,
  GraphLabel,
  GraphQuery,
  GraphResponse,
  MetricMeasure,
  SplitMode,
  SplitType,
  Universe
} from '../helpers/dtos';
import {
  getAnomalyMetricMeasure,
  getAnomalyOutlierType,
  getAnomalyNumNodes,
  getAnomalyStartDate,
  getAnomalyEndTime
} from '../helpers/utils';
import {
  ALL,
  filterDurations,
  MAX_OUTLIER_NUM_NODES,
  ALL_REGIONS,
  ALL_ZONES,
  QUERY_REQUEST_GRAPHS,
  TABLET_REQUEST_GRAPHS,
  ASH_GROUPBY_VALUES
} from '../helpers/constants';
import { useHelperStyles } from './styles';

import { ReactComponent as LoadingIcon } from '../assets/loading.svg';

export interface SecondaryDashboardDataProps {
  anomalyData: Anomaly | null;
  universeUuid: string;
  universeData: Universe | any;
  appName: AppName;
  graphParams: GraphQuery[] | null;
  timezone?: string;
  recommendationMetrics: any;
  apiUrl: string;
  universeQueryData: any;
  isHistoricalQueryView: boolean;
}

export const SecondaryDashboardData = ({
  universeUuid,
  universeData,
  anomalyData,
  graphParams,
  appName,
  timezone,
  recommendationMetrics,
  apiUrl,
  universeQueryData,
  isHistoricalQueryView
}: SecondaryDashboardDataProps) => {
  const helperClasses = useHelperStyles();
  // Get default values to be populated on page
  const anomalyMetricMeasure = getAnomalyMetricMeasure(anomalyData!);
  const anomalyOutlierType = getAnomalyOutlierType(anomalyData!);
  const anomalyOutlierNumNodes = getAnomalyNumNodes(anomalyData!);
  const anomalyStartDate = getAnomalyStartDate(anomalyData!);
  const anomalyEndDate = getAnomalyEndTime(anomalyData!);
  const today = new Date();
  const yesterday = new Date(today);
  yesterday.setDate(yesterday.getDate() - 1);

  // State variables
  const [shouldViewAdvancedData, setShouldViewAdvancedData] = useState<boolean>(false);
  // const [currentTab, setCurrentTab] = useState<SecondaryDashboardTabs>(DEFAULT_TAB);
  const [graphQueryParams, setGraphQueryParams] = useState<any>(graphParams);
  const [queryViewParams, setAdvancedViewParams] = useState<any>([]);
  const [clusterRegionItem, setClusterRegionItem] = useState<string>(ALL_REGIONS);
  const [zoneNodeItem, setZoneNodeItem] = useState<string>(ALL_ZONES);
  const [isPrimaryCluster, setIsPrimaryCluster] = useState<boolean>(true);
  const [cluster, setCluster] = useState<string>(ALL);
  const [region, setRegion] = useState<string>(ALL);
  const [zone, setZone] = useState<string>(ALL);
  const [node, setNode] = useState<string>(ALL);
  const [metricMeasure, setMetricMeasure] = useState<string>(anomalyMetricMeasure);
  const [outlierType, setOutlierType] = useState<SplitMode>(anomalyOutlierType);
  const [filterDuration, setFilterDuration] = useState<string>(
    anomalyStartDate ? filterDurations[filterDurations.length - 1].label : filterDurations[0].label
  );
  const [numNodes, setNumNodes] = useState<number>(anomalyOutlierNumNodes);
  const [startDateTime, setStartDateTime] = useState<Date>(anomalyStartDate ?? yesterday);
  const [endDateTime, setEndDateTime] = useState<Date>(anomalyEndDate ?? today);
  const [RCAGraphData, setRCAGraphData] = useState<any>(null);
  const [queryGraphData, setQueryGraphData] = useState<any>(null);

  // Check previous props
  const previousMetricMeasure = usePrevious(metricMeasure);

  // Make use of useMutation to call fetchGraphs and onSuccess of it, ensure to set setChartData
  useUpdateEffect(() => {
    if (previousMetricMeasure && previousMetricMeasure !== metricMeasure) {
      setRCAGraphData(null);
    }

    const graphParamsCopy = JSON.parse(JSON.stringify(graphQueryParams));

    const formattedStartDate = startDateTime?.toISOString();
    const formattedEndDate = endDateTime?.toISOString();
    if (!shouldViewAdvancedData && !isHistoricalQueryView) {
      graphParamsCopy?.map((params: GraphQuery) => {
        const settings = params.settings;
        const filters = params.filters;
        let start = params.start;
        let end = params.end;

        if (start !== formattedStartDate) {
          params.start = formattedStartDate;
        }

        if (end !== formattedEndDate) {
          params.end = formattedEndDate;
        }
        if (isNonEmptyString(cluster) && cluster !== ALL) {
          filters.clusterUuid = [cluster];
        }
        if (isNonEmptyString(region) && region !== ALL) {
          filters.regionCode = [region];
        }
        if (isNonEmptyString(zone) && zone !== ALL) {
          filters.azCode = [zone];
        }
        if (isNonEmptyString(node) && node !== ALL) {
          filters.instanceName = [node];
        }
        settings.returnAggregatedValue = metricMeasure !== MetricMeasure.OVERALL;
        settings.splitType =
          metricMeasure === MetricMeasure.OVERALL
            ? SplitType.NONE
            : metricMeasure === MetricMeasure.OUTLIER
            ? SplitType.NODE
            : SplitType.TABLE;
        settings.splitMode = metricMeasure === MetricMeasure.OVERALL ? SplitMode.NONE : outlierType;
        settings.splitCount = numNodes;
        return settings;
      });

      const actualQueryParams = graphParamsCopy ?? graphQueryParams;
      setGraphQueryParams(actualQueryParams);
      fetchAnomalyGraphs.mutate(actualQueryParams);
    } else if (shouldViewAdvancedData && !isHistoricalQueryView) {
      // In case of Tablet Anomaly, we need to fetch the graphs ASH TServer twice
      // One graph with groupBy "TabletId", pass aggregatedLineSettings and
      // one without aggregatedLineSettings and groupBy "waitEventType"
      // TODO: Discuss with Aleks to see how backend can send back a param saying if
      // groupBy dropdown needs to be shown or not
      const REQUEST_GRAPHS =
        anomalyData?.category === AnomalyCategory.SQL
          ? QUERY_REQUEST_GRAPHS
          : TABLET_REQUEST_GRAPHS;

      // Advanced Query View payload
      const advancedQueryPayloadCopy: GraphQuery[] = [];
      REQUEST_GRAPHS.map((requestGraph: string, requestIdx: number) => {
        const advancedQueryPayload: GraphQuery = {
          start: formattedStartDate,
          end: formattedEndDate,
          name: requestGraph,
          filters: {
            universeUuid: [universeUuid],
            queryId: anomalyData?.mainGraphs[0]?.filters?.queryId
          },
          settings: {
            splitMode: metricMeasure === MetricMeasure.OVERALL ? SplitMode.NONE : outlierType,
            splitType:
              metricMeasure === MetricMeasure.OVERALL
                ? SplitType.NONE
                : metricMeasure === MetricMeasure.OUTLIER
                ? SplitType.NODE
                : SplitType.TABLE,
            splitCount: numNodes,
            returnAggregatedValue: metricMeasure !== MetricMeasure.OVERALL,
            aggregatedValueFunction: 'AVG'
          }
        };

        if (anomalyData?.category === AnomalyCategory.SQL) {
          advancedQueryPayload.filters.queryId = anomalyData?.mainGraphs[0]?.filters?.queryId;
          if (requestGraph.startsWith('active_session_history')) {
            advancedQueryPayload.groupBy = [GraphLabel.WAIT_EVENT_TYPE];
          }
        }
        if (anomalyData?.category === AnomalyCategory.DB) {
          advancedQueryPayload.filters.dbName = anomalyData?.mainGraphs[0]?.filters?.dbName;
          advancedQueryPayload.filters.tableName = anomalyData?.mainGraphs[0]?.filters?.tableName;
          advancedQueryPayload.filters.tabletId = anomalyData?.mainGraphs[0]?.filters?.tabletId;
          // Do not pass aggregatedLineSettings for the last graph
          if (REQUEST_GRAPHS.length - 1 !== requestIdx) {
            advancedQueryPayload.aggregatedLineSettings =
              anomalyData?.mainGraphs[0]?.aggregatedLineSettings;
          }

          REQUEST_GRAPHS.length - 1 === requestIdx
            ? (advancedQueryPayload.groupBy = [GraphLabel.WAIT_EVENT_TYPE])
            : (advancedQueryPayload.groupBy = [GraphLabel.TABLET_ID]);
        }
        advancedQueryPayloadCopy.push(advancedQueryPayload);
      });
      setAdvancedViewParams(advancedQueryPayloadCopy);
      fetchAdvancedViewGraphs.mutate(advancedQueryPayloadCopy);
    }
  }, [
    numNodes,
    metricMeasure,
    filterDuration,
    outlierType,
    node,
    zone,
    region,
    cluster,
    endDateTime,
    startDateTime,
    anomalyData?.graphStartTime,
    anomalyData?.graphEndTime,
    shouldViewAdvancedData
  ]);

  const fetchAnomalyGraphs = useMutation(
    (params: any) => TroubleshootAPI.fetchGraphs(universeUuid, params, apiUrl),
    {
      onSuccess: (data: GraphResponse[]) => {
        setRCAGraphData(data);
      },
      onError: (error: any) => {
        console.error('Failed to fetch RCA graphs', error);
      }
    }
  );

  const fetchAdvancedViewGraphs = useMutation(
    (params: any) => TroubleshootAPI.fetchGraphs(universeUuid, params, apiUrl),
    {
      onSuccess: (data: GraphResponse[]) => {
        setQueryGraphData(data);
      },
      onError: (error: any) => {
        console.error('Failed to fetch Query graphs', error);
      }
    }
  );

  const onSelectAshLabel = (groupByLabel: string, metricData: any, isAdvancedView = false) => {
    let groupBy = ASH_GROUPBY_VALUES.WAIT_EVENT_COMPONENT;
    if (groupByLabel === 'Class') {
      groupBy = ASH_GROUPBY_VALUES.WAIT_EVENT_CLASS;
    } else if (groupByLabel === 'Type') {
      groupBy = ASH_GROUPBY_VALUES.WAIT_EVENT_TYPE;
    } else if (groupByLabel === 'Event') {
      groupBy = ASH_GROUPBY_VALUES.WAIT_EVENT;
    } else if (groupByLabel === 'Query') {
      groupBy = ASH_GROUPBY_VALUES.QUERY;
    } else if (groupByLabel === 'Client IP') {
      groupBy = ASH_GROUPBY_VALUES.CLIENT_NODE_IP;
    }

    if (isAdvancedView) {
      const queryViewParamsCopy = JSON.parse(JSON.stringify(queryViewParams));
      let ashGraph = null;
      if (anomalyData?.category === AnomalyCategory.DB) {
        const filteredParamsCopy = queryViewParamsCopy?.filter(
          (queryParams: GraphQuery) => queryParams.name === metricData?.name
        );
        if (filteredParamsCopy.length >= 2) {
          // Ensure to apply groupBy only on the last graph when there are 2 or more active_session_history_tserver graphs
          const filteredMetricData = filteredParamsCopy.find(
            (queryParams: GraphQuery, idx: number) =>
              idx === filteredParamsCopy.length - 1 && queryParams.name === metricData?.name
          );
          ashGraph = filteredMetricData;
        } else {
          ashGraph = queryViewParamsCopy?.find(
            (queryParams: GraphQuery, idx: number) =>
              idx > 0 && queryParams.name === metricData?.name
          );
        }
      } else {
        ashGraph = queryViewParamsCopy?.find(
          (queryParams: GraphQuery, idx: number) => idx > 0 && queryParams.name === metricData?.name
        );
      }

      if (ashGraph) {
        ashGraph.groupBy = [groupBy];
      }
      queryViewParamsCopy.map((obj: any, paramsIdx: number) => {
        if (paramsIdx > 0 && obj.name === metricData?.name) {
          return ashGraph;
        }
        return obj;
      });
      setAdvancedViewParams(queryViewParamsCopy);
      fetchAdvancedViewGraphs.mutate(queryViewParamsCopy);
    } else {
      const graphParamsCopy = JSON.parse(JSON.stringify(graphQueryParams));
      const ashGraph = graphParamsCopy?.find(
        (queryParams: GraphQuery, idx: number) => idx > 0 && queryParams.name === metricData?.name
      );
      if (ashGraph) {
        ashGraph.groupBy = [groupBy];
      }
      graphParamsCopy.map((obj: any, paramsIdx: number) => {
        if (paramsIdx > 0 && obj.name === metricData?.name) {
          return ashGraph;
        }
        return obj;
      });
      setGraphQueryParams(graphParamsCopy);
      fetchAnomalyGraphs.mutate(graphParamsCopy);
    }
  };

  useEffect(() => {
    fetchAnomalyGraphs.mutate(graphQueryParams);
  }, []);

  const onSplitTypeSelected = (metricMeasure: string) => {
    setMetricMeasure(metricMeasure);
  };

  const onOutlierTypeSelected = (outlierType: SplitMode) => {
    setOutlierType(outlierType);
  };

  const onNumNodesChanged = (numNodes: number) => {
    setNumNodes(numNodes > MAX_OUTLIER_NUM_NODES ? MAX_OUTLIER_NUM_NODES : numNodes);
  };

  const onSelectedFilterDuration = (filterDuration: string) => {
    setFilterDuration(filterDuration);
  };

  const onClusterRegionSelected = (
    isCluster: boolean,
    isRegion: boolean,
    selectedOption: string,
    isPrimaryCluster: boolean
  ) => {
    setIsPrimaryCluster(isPrimaryCluster);
    if (selectedOption === ALL_REGIONS) {
      setClusterRegionItem(ALL_REGIONS);
      setCluster(ALL);
      setRegion(ALL);
    }

    if (isCluster || isRegion) {
      setClusterRegionItem(selectedOption);

      if (isCluster) {
        setCluster(selectedOption);
        setRegion('');
      }

      if (isRegion) {
        setRegion(selectedOption);
        setCluster('');
      }
    }
  };

  const onZoneNodeSelected = (isZone: boolean, isNode: boolean, selectedOption: string) => {
    if (selectedOption === ALL_ZONES) {
      setZoneNodeItem(ALL_ZONES);
      setZone(ALL);
      setNode(ALL);
    }

    if (isZone || isNode) {
      setZoneNodeItem(selectedOption);

      if (isZone) {
        setZone(selectedOption);
        setNode('');
      }

      if (isNode) {
        setZone('');
        setNode(selectedOption);
      }
    }
  };

  const onStartDateChange = (e: ChangeEvent<HTMLInputElement>) => {
    setStartDateTime(new Date(e.target.value));
  };

  const onEndDateChange = (e: ChangeEvent<HTMLInputElement>) => {
    setEndDateTime(new Date(e.target.value));
  };

  const onViewQueryData = () => {
    setShouldViewAdvancedData(!shouldViewAdvancedData);
  };

  return (
    <Box>
      <SecondaryDashboardHeader
        appName={appName}
        universeData={universeData}
        clusterRegionItem={clusterRegionItem}
        zoneNodeItem={zoneNodeItem}
        isPrimaryCluster={isPrimaryCluster}
        viewQueryData={shouldViewAdvancedData}
        cluster={cluster}
        region={region}
        metricMeasure={metricMeasure}
        filterDuration={filterDuration}
        numNodes={numNodes}
        startDateTime={startDateTime}
        endDateTime={endDateTime}
        anomalyData={anomalyData}
        shouldShowAdvancedData={
          anomalyData?.category === AnomalyCategory.SQL ||
          anomalyData?.category === AnomalyCategory.DB
        }
        onZoneNodeSelected={onZoneNodeSelected}
        onClusterRegionSelected={onClusterRegionSelected}
        onOutlierTypeSelected={onOutlierTypeSelected}
        onSplitTypeSelected={onSplitTypeSelected}
        onNumNodesChanged={onNumNodesChanged}
        onSelectedFilterDuration={onSelectedFilterDuration}
        onStartDateChange={onStartDateChange}
        onEndDateChange={onEndDateChange}
        onViewQueryData={onViewQueryData}
        timezone={timezone}
      />
      {!isHistoricalQueryView &&
        (shouldViewAdvancedData ? (
          fetchAdvancedViewGraphs.isLoading ? (
            <Box className={helperClasses.loadingBox}>
              <LoadingIcon className={clsx(helperClasses.icon, helperClasses.inProgressIcon)} />
            </Box>
          ) : (
            <QueryDataView
              queryId={anomalyData?.mainGraphs[0]?.filters?.queryId}
              metricMeasure={metricMeasure}
              anomalyData={anomalyData}
              appName={appName}
              timezone={timezone}
              universeQueryData={universeQueryData}
              graphData={queryGraphData}
              graphQueryParams={queryViewParams}
              onSelectAshLabel={onSelectAshLabel}
            />
          )
        ) : fetchAnomalyGraphs.isLoading ? (
          <Box className={helperClasses.loadingBox}>
            <LoadingIcon className={clsx(helperClasses.icon, helperClasses.inProgressIcon)} />
          </Box>
        ) : (
          <RCAView
            metricMeasure={metricMeasure}
            anomalyData={anomalyData}
            appName={appName}
            timezone={timezone}
            universeQueryData={universeQueryData}
            graphQueryParams={graphQueryParams}
            recommendationMetrics={recommendationMetrics}
            graphData={RCAGraphData}
            onSelectAshLabel={onSelectAshLabel}
          />
        ))}
    </Box>
  );
};
