import { useState } from 'react';
import { useQuery } from 'react-query';
import { Box, Typography } from '@material-ui/core';
import _ from 'lodash';
import clsx from 'clsx';
import { Link } from 'react-router';
import { isNonEmptyString, YBButton, YBErrorIndicator } from '@yugabytedb/ui-components';
import { SecondaryDashboardData } from './SecondaryDashboardData';
import { QUERY_KEY, TroubleshootAPI } from '../api';
import { Anomaly, AppName, GraphQuery, Universe } from '../helpers/dtos';
import { getGraphRequestParams, getRecommendationMetricsMap } from '../helpers/utils';
import { useHelperStyles } from './styles';

import { ReactComponent as LoadingIcon } from '../assets/loading.svg';

export interface SecondaryDashboardEntryProps {
  universeUuid: string;
  hideHeader?: boolean;
  troubleshootUuid: string | null;
  // TODO: any should be replaced with YBM Node Response
  universeData: Universe | any;
  appName: AppName;
  timezone?: string;
  apiUrl: string;
  isHistoricalQueryView?: boolean;
  onSelectedIssue?: (troubleshootUuid: string | null) => void;
}

export const SecondaryDashboardEntry = ({
  universeUuid,
  troubleshootUuid,
  universeData,
  appName,
  timezone,
  hideHeader = false,
  apiUrl,
  isHistoricalQueryView = false,
  onSelectedIssue
}: SecondaryDashboardEntryProps) => {
  const helperClasses = useHelperStyles();

  const [userSelectedAnomaly, setUserSelectedAnomaly] = useState<Anomaly | null>(null);
  const [graphRequestParams, setGraphRequestParams] = useState<GraphQuery[] | null>(null);
  const [recommendationMetrics, setRecommendationMetrics] = useState<any>(null);
  const [universeQueryData, setUniverseQueryData] = useState<any>(null);

  const { isLoading, isError, isIdle } = useQuery(
    [QUERY_KEY.fetchAnomalies, universeUuid],
    () => TroubleshootAPI.fetchAnomaliesById(universeUuid, troubleshootUuid!, apiUrl),
    {
      onSuccess: (anomalyData: Anomaly) => {
        setUserSelectedAnomaly(anomalyData);
        setGraphRequestParams(getGraphRequestParams(anomalyData));
        setRecommendationMetrics(getRecommendationMetricsMap(anomalyData));
      },
      onError: (error: any) => {
        console.error('Failed to fetch Anomalies', error);
      },
      enabled: !isHistoricalQueryView && isNonEmptyString(troubleshootUuid!)
    }
  );

  const { isLoading: universeQueriesLoading, isError: universeQueriesError } = useQuery(
    [QUERY_KEY.fetchQueries, universeUuid],
    () => TroubleshootAPI.fetchQueries(universeUuid, apiUrl),
    {
      onSuccess: (data) => {
        setUniverseQueryData(data);
      },
      onError: (error: any) => {
        setUniverseQueryData([]);
        console.error('Failed to fetch queries', error);
      },
      enabled: !isHistoricalQueryView && isNonEmptyString(troubleshootUuid!)
    }
  );

  if (isLoading || universeQueriesLoading) {
    return (
      <Box className={helperClasses.loadingBox}>
        <LoadingIcon className={clsx(helperClasses.icon, helperClasses.inProgressIcon)} />
      </Box>
    );
  }

  if (isError || universeQueriesError || (isIdle && userSelectedAnomaly === undefined)) {
    return <YBErrorIndicator customErrorMessage={'Failed to fetch anomalies list'} />;
  }

  const routeToPrimary = () => {
    onSelectedIssue?.(null);
  };

  return (
    <Box>
      {!hideHeader && (
        <Typography variant="h2" className="content-title">
          {appName === AppName.YBA ? (
            <Link to={`/universes/${universeUuid}/troubleshoot`}>
              <Typography variant="h3">{'Troubleshoot'}</Typography>
            </Link>
          ) : (
            <Box>
              <YBButton variant="pill" data-testid="BtnAddIPList" onClick={() => routeToPrimary()}>
                {'Troubleshoot'}
              </YBButton>
            </Box>
          )}
        </Typography>
      )}
      {userSelectedAnomaly && universeQueryData && (
        <SecondaryDashboardData
          apiUrl={apiUrl}
          anomalyData={userSelectedAnomaly}
          universeData={universeData}
          universeUuid={universeUuid}
          appName={appName}
          timezone={timezone}
          graphParams={graphRequestParams}
          recommendationMetrics={recommendationMetrics}
          universeQueryData={universeQueryData}
          isHistoricalQueryView={isHistoricalQueryView}
        />
      )}
    </Box>
  );
};
