import { useState } from 'react';
import { useQuery } from 'react-query';
import { Box } from '@material-ui/core';
import clsx from 'clsx';
import { YBErrorIndicator } from '@yugabytedb/ui-components';
import { PrimaryDashboardData } from './PrimaryDashboardData';
import { TroubleshootAPI, QUERY_KEY } from '../api';
import { Anomaly, AppName } from '../helpers/dtos';
import { useHelperStyles } from './styles';

import { ReactComponent as LoadingIcon } from '../assets/loading.svg';
import { useUpdateEffect } from 'react-use';

interface TroubleshootAdvisorProps {
  universeUuid: string;
  appName: AppName;
  timezone?: string;
  apiUrl: string;
  onSelectedIssue?: (troubleshootUuid: string) => void;
}

export const TroubleshootAdvisor = ({
  universeUuid,
  appName,
  timezone,
  apiUrl,
  onSelectedIssue
}: TroubleshootAdvisorProps) => {
  const helperClasses = useHelperStyles();
  const classes = useHelperStyles();

  const [anomalyList, setAnomalyList] = useState<Anomaly[] | null>(null);
  const [startDateTime, setDateStartTime] = useState<Date | null>(null);
  const [endDateTime, setDateEndTime] = useState<Date | null>(null);

  const { isLoading, isError, isIdle, refetch: anomaliesRefetch } = useQuery(
    [QUERY_KEY.fetchAnomalies, universeUuid],
    () => TroubleshootAPI.fetchAnomalies(universeUuid, startDateTime, endDateTime, apiUrl),
    {
      enabled: anomalyList === null,
      onSuccess: (data: Anomaly[]) => {
        setAnomalyList(data);
      },
      onError: (error: any) => {
        console.error('Failed to fetch anomalies', error);
      }
    }
  );

  const onFilterByDate = (startDate: any, endDate: any) => {
    setDateStartTime(startDate);
    setDateEndTime(endDate);
  };

  useUpdateEffect(() => {
    anomaliesRefetch();
  }, [startDateTime, endDateTime]);

  if (isLoading) {
    return (
      <Box className={classes.loadingBox}>
        <LoadingIcon className={clsx(classes.icon, classes.inProgressIcon)} />
      </Box>
    );
  }
  if (isError || (isIdle && anomalyList === null)) {
    return (
      <Box className={helperClasses.recommendation}>
        <YBErrorIndicator
          customErrorMessage={'Failed to fetch anomalies list, please try again.'}
        />
      </Box>
    );
  }
  return (
    <Box sx={{ width: '100%' }}>
      <Box sx={{ borderBottom: 1, borderColor: 'divider' }}>
        <Box m={2}>
          <PrimaryDashboardData
            anomalyData={anomalyList}
            appName={appName}
            timezone={timezone}
            universeUuid={universeUuid}
            apiUrl={apiUrl}
            onFilterByDate={onFilterByDate}
            onSelectedIssue={onSelectedIssue}
          />
        </Box>
      </Box>
    </Box>
  );
};
