import { useState } from 'react';
import { Box, makeStyles } from '@material-ui/core';
import clsx from 'clsx';
import {
  YBPanelItem,
  YBErrorIndicator,
  YBButton,
  isNonEmptyString
} from '@yugabytedb/ui-components';
import { MetadataFields, Universe } from '../../helpers/dtos';
import { useQuery, useQueryClient } from 'react-query';
import { TroubleshootAPI, QUERY_KEY } from '../../api';
import { UniverseListMetadata } from './UniverseListMetadata';
import { useHelperStyles } from '../styles';

import { ReactComponent as LoadingIcon } from '../../assets/loading.svg';

interface TroubleshootConfigurationProps {
  apiUrl: string;
  customerUuid: string;
}

const useStyles = makeStyles((theme) => ({
  refreshButton: {
    width: '120px',
    marginTop: theme.spacing(2),
    alignSelf: 'end'
  },
  flexColumn: {
    display: 'flex',
    flexDirection: 'column'
  },
  noUniverses: {
    alignSelf: 'center'
  }
}));

export const TroubleshootConfiguration = ({
  apiUrl,
  customerUuid
}: TroubleshootConfigurationProps) => {
  const classes = useStyles();
  const helperClasses = useHelperStyles();
  const [metadata, setMetadata] = useState<MetadataFields[]>([]);
  const [universeList, setUniverseList] = useState<Universe[]>([]);
  const queryClient = useQueryClient();

  const {
    isIdle: isFetchMetadataIdle,
    isLoading: isFetchMetadataLoading,
    isError: isFetchMetadataError,
    refetch: metadataRefetch
  } = useQuery(
    QUERY_KEY.fetchUniverseMetadataList,
    () => TroubleshootAPI.fetchUniverseMetadataList(apiUrl, customerUuid),
    {
      onSuccess: (data: MetadataFields[]) => {
        setMetadata(data);
      }
    }
  );

  const {
    isIdle: isFetchUniverseIdle,
    isLoading: isFetchUniverseLoading,
    isError: isFetchUniverseError,
    refetch: universeListRefetch
  } = useQuery(
    QUERY_KEY.fetchUniverseListDetails,
    () => TroubleshootAPI.fetchUniverseListDetails(apiUrl),
    {
      onSuccess: (data: Universe[]) => {
        setUniverseList(data);
      }
    }
  );

  if (isFetchMetadataError || isFetchUniverseError) {
    <YBErrorIndicator customErrorMessage={'Unable to fetch universe details'} />;
  }

  if (
    isFetchMetadataLoading ||
    isFetchUniverseLoading ||
    (isFetchMetadataIdle && metadata === undefined) ||
    (isFetchUniverseIdle && universeList === undefined)
  ) {
    return (
      <Box className={helperClasses.loadingBox}>
        <LoadingIcon className={clsx(helperClasses.icon, helperClasses.inProgressIcon)} />
      </Box>
    );
  }

  const onActionPerformed = () => {
    const getLatestMetadata = async () => {
      queryClient.invalidateQueries(QUERY_KEY.fetchUniverseMetadataList);
      queryClient.invalidateQueries(QUERY_KEY.fetchUniverseListDetails);
      await metadataRefetch();
      await universeListRefetch();
    };
    getLatestMetadata();
  };

  return (
    <YBPanelItem
      body={
        <Box ml={1} mr={1} mt={1} className={classes.flexColumn}>
          <YBButton
            variant="secondary"
            size="medium"
            onClick={() => {
              onActionPerformed();
            }}
            className={classes.refreshButton}
          >
            {'Refresh'}
          </YBButton>

          {isNonEmptyString(apiUrl) && metadata.length > 0 ? (
            <>
              <Box mt={2}>
                <UniverseListMetadata
                  apiUrl={apiUrl}
                  customerUuid={customerUuid}
                  metadata={metadata}
                  universeList={universeList}
                  onActionPerformed={onActionPerformed}
                />
              </Box>
            </>
          ) : (
            <Box className={classes.noUniverses}>
              {isNonEmptyString(apiUrl)
                ? 'Currently, there are no universes attached to troubleshooting service'
                : 'Not registered with Troubleshooting Platform service. Please register and then attach universes.'}
            </Box>
          )}
        </Box>
      }
    />
  );
};
