import { isEmptyArray, isNonEmptyArray, isNonEmptyObject } from '@yugabytedb/ui-components';
import { ASH, MIN_OUTLIER_NUM_NODES } from './constants';
import {
  Anomaly,
  AppName,
  GraphLabel,
  GraphMetadata,
  GraphQuery,
  GraphType,
  MetricMeasure,
  RCAGuideline,
  SplitMode,
  SplitType,
  TroubleshootingRecommendations
} from './dtos';

let index = 0;

export const formatData = (data: any, appName: AppName) => {
  const formattedData =
    appName === AppName.YBA ? formatUniverseDetails(data) : formatClusterDetails(data);
  return formattedData;
};

export const getAnomalyMetricMeasure = (anomalyData: Anomaly) => {
  let metricMeasure = MetricMeasure.OVERALL;
  if (
    anomalyData?.defaultSettings?.splitMode === SplitType.NONE &&
    anomalyData?.defaultSettings?.splitType === SplitType.NONE
  ) {
    metricMeasure = MetricMeasure.OVERALL;
  }

  if (
    anomalyData?.defaultSettings?.splitMode === SplitMode.TOP ||
    anomalyData?.defaultSettings?.splitMode === SplitMode.BOTTOM
  ) {
    metricMeasure =
      anomalyData?.defaultSettings?.splitType === SplitType.NODE
        ? MetricMeasure.OUTLIER
        : MetricMeasure.OUTLIER_TABLES;
  }

  return metricMeasure;
};

export const getAnomalyOutlierType = (anomalyData: Anomaly) => {
  let outlierType: SplitMode = SplitMode.TOP;
  if (anomalyData?.defaultSettings?.splitMode === SplitMode.TOP) {
    outlierType = SplitMode.TOP;
  } else if (anomalyData?.defaultSettings?.splitMode === SplitMode.BOTTOM) {
    outlierType = SplitMode.BOTTOM;
  }

  return outlierType;
};

export const getAnomalyNumNodes = (anomalyData: Anomaly) => {
  let numNodes = MIN_OUTLIER_NUM_NODES;
  if (anomalyData?.defaultSettings?.splitCount > 0) {
    numNodes = anomalyData?.defaultSettings?.splitCount;
  }
  return numNodes;
};

export const getAnomalyStartDate = (anomalyData: Anomaly) => {
  return anomalyData?.graphStartTime ? new Date(anomalyData.graphStartTime!) : null;
};

export const getAnomalyEndTime = (anomalyData: Anomaly) => {
  return anomalyData?.graphEndTime ? new Date(anomalyData.graphEndTime!) : null;
};

export const formatUniverseDetails = (universeData: any) => {
  const primaryClusterMapping: any = new Map();
  const asyncClusterMapping: any = new Map();

  const primaryZoneMapping: any = new Map();
  const asyncZoneMapping: any = new Map();
  const nodeIPMapping: any = new Map();

  const universeDetails = universeData?.universeDetails;
  if (universeDetails) {
    const nodeDetailsSet = universeDetails.nodeDetailsSet;
    const clusters = universeDetails.clusters;
    const primaryCluster = getPrimaryCluster(clusters);
    const asyncCluster = getReadOnlyCluster(clusters);

    const primaryClusterRegionList = primaryCluster.placementInfo.cloudList[0].regionList;
    const asyncClusterRegionList = asyncCluster?.placementInfo?.cloudList[0]?.regionList;

    const primaryClusterUuid = primaryCluster.uuid;
    const asyncClusterUuid = asyncCluster?.uuid;
    const primaryClusterRegions = primaryClusterRegionList.map((region: any) => region.code);
    const asyncClusterRegions = asyncClusterRegionList?.map((region: any) => region.code);

    primaryClusterMapping.set('Primary', {
      cluster: 'Primary',
      regions: primaryClusterRegions,
      uuid: primaryClusterUuid
    });

    if (asyncCluster) {
      asyncClusterMapping.set('Async', {
        cluster: 'Read Replica',
        regions: asyncClusterRegions,
        uuid: asyncClusterUuid
      });
    }

    for (const { cloudInfo, placementUuid, nodeName } of nodeDetailsSet) {
      nodeIPMapping.set(nodeName, {
        nodeName: nodeName,
        ip: cloudInfo.private_ip
      });
      if (placementUuid === primaryClusterUuid) {
        primaryZoneMapping.set(cloudInfo.az, {
          zoneName: cloudInfo.az,
          nodeNames: getNodesBasedOnZonesYBA(nodeDetailsSet, cloudInfo.az, primaryClusterUuid),
          isReadReplica: false,
          regionName: cloudInfo.region
        });
      } else {
        asyncZoneMapping.set(cloudInfo.az, {
          zoneName: cloudInfo.az,
          nodeNames: getNodesBasedOnZonesYBA(nodeDetailsSet, cloudInfo.az, asyncClusterUuid),
          isReadReplica: true,
          regionName: cloudInfo.region
        });
      }
    }
  }

  return {
    primaryZoneMapping,
    asyncZoneMapping,
    primaryClusterMapping,
    asyncClusterMapping,
    nodeIPMapping
  };
};

export const formatClusterDetails = (nodeData: any) => {
  const primaryClusterMapping: any = new Map();
  const asyncClusterMapping: any = new Map();

  const primaryZoneMapping: any = new Map();
  const asyncZoneMapping: any = new Map();

  const nodeDetails = nodeData?.data;
  if (nodeDetails) {
    for (const { cloud_info, is_read_replica } of nodeDetails) {
      !is_read_replica
        ? primaryClusterMapping.set('Primary', {
            cluster: 'Primary',
            regions: getRegionsBasedOnCluster(nodeDetails, false)
          })
        : asyncClusterMapping.set('Async', {
            cluster: 'Read Replica',
            regions: getRegionsBasedOnCluster(nodeDetails, true)
          });

      !is_read_replica
        ? primaryZoneMapping.set(cloud_info.zone, {
            zoneName: cloud_info.zone,
            nodeNames: getNodesBasedOnZones(nodeDetails, cloud_info.zone, false),
            isReadReplica: is_read_replica,
            regionName: cloud_info.region
          })
        : asyncZoneMapping.set(cloud_info.zone, {
            zoneName: cloud_info.zone,
            nodeNames: getNodesBasedOnZones(nodeDetails, cloud_info.zone, true),
            isReadReplica: is_read_replica,
            regionName: cloud_info.region
          });
    }
  }

  return {
    primaryZoneMapping,
    asyncZoneMapping,
    primaryClusterMapping,
    asyncClusterMapping
  };
};

export const getNodesBasedOnZones = (
  nodeDetailsData: any,
  zoneName: string,
  isReadReplica: boolean
) => {
  const nodeNames: Array<string> = [];
  for (const { name, cloud_info, is_read_replica } of nodeDetailsData) {
    if (zoneName === cloud_info.zone && is_read_replica === isReadReplica) nodeNames.push(name);
  }
  return nodeNames;
};

export const getNodesBasedOnZonesYBA = (nodeDetailsData: any, zoneName: string, uuid: string) => {
  const nodeNames: Array<string> = [];
  for (const { nodeName, cloudInfo, placementUuid } of nodeDetailsData) {
    if (zoneName === cloudInfo.az && placementUuid === uuid) nodeNames.push(nodeName);
  }
  return nodeNames;
};

export const getRegionsBasedOnCluster = (nodeDetailsData: any, isReadReplica: boolean) => {
  const regions: Array<string> = [];
  for (const { cloud_info, is_read_replica } of nodeDetailsData) {
    if (is_read_replica === isReadReplica && !regions.includes(cloud_info.region)) {
      regions.push(cloud_info.region);
    }
  }
  return regions;
};

export const getFilteredItems = (
  zoneToNodesMap: any,
  isRegionChanged: boolean,
  isPrimaryCluster: boolean,
  filterParam: string
) => {
  const zoneToNodesList = Array.from(zoneToNodesMap);
  const filteredList = zoneToNodesList.filter((item: any) => {
    return isRegionChanged
      ? item[1].regionName === filterParam
      : isPrimaryCluster
      ? !item[1].isReadReplica
      : item[1].isReadReplica;
  });
  const filteredMap = filteredList
    ? new Map(filteredList.map((obj: any) => [obj[0], obj[1]]))
    : new Map();
  return filteredMap;
};

export const getPrimaryCluster = (clusters: any) => {
  if (isNonEmptyArray(clusters)) {
    const foundClusters = clusters.filter((cluster: any) => cluster.clusterType === 'PRIMARY');
    if (foundClusters.length === 1) {
      return foundClusters[0];
    }
  }
  return null;
};

export function getReadOnlyCluster(clusters: any) {
  if (isNonEmptyArray(clusters)) {
    const foundClusters = clusters.filter((cluster: any) => cluster.clusterType === 'ASYNC');
    if (foundClusters.length === 1) {
      return foundClusters[0];
    }
  }
  return null;
}

export const getGraphRequestParams = (
  anomalyData: Anomaly,
  startDate = null,
  endDate = null,
  splitType = null,
  splitMode = null,
  splitNum = null
) => {
  const mainGraphRequest = anomalyData?.mainGraphs.map((graph: GraphMetadata) => {
    const request: any = {};
    request.name = graph.name;
    request.filters = graph.filters;
    request.start = anomalyData.graphStartTime;
    request.end = anomalyData.graphEndTime;
    request.settings = anomalyData.defaultSettings;
    request.aggregatedLineSettings = graph.aggregatedLineSettings;
    request.groupBy = graph.groupBy;
    return request;
  });

  const nestedRequest = anomalyData?.rcaGuidelines?.map((rca: RCAGuideline) => {
    return rca.troubleshootingRecommendations?.map(
      (recommendation: TroubleshootingRecommendations) => {
        return recommendation.supportingGraphs?.map((graph: GraphMetadata) => {
          const request: any = {};
          request.name = graph.name;
          request.filters = graph.filters;
          request.start = anomalyData.graphStartTime;
          request.end = anomalyData.graphEndTime;
          request.settings = anomalyData.defaultSettings;
          request.aggregatedLineSettings = graph.aggregatedLineSettings;
          request.groupBy = graph.groupBy;
          return request;
        });
      }
    );
  });

  const flattenedRequest = nestedRequest?.flat(2);
  const supportingGraphRequest = flattenedRequest?.filter((request) => isNonEmptyObject(request));

  const requestParamsList = [...mainGraphRequest, ...supportingGraphRequest];
  return requestParamsList;
};

export const getRecommendationMetricsMap = (anomalyData: Anomaly) => {
  index = 0;
  if (!anomalyData) {
    return [];
  }

  const nestedRecommendationMetrics = anomalyData?.rcaGuidelines?.map(
    (rca: RCAGuideline, rcaIndex: number) => {
      const params: any = {
        cause: '',
        name: [],
        // Determines if we need to show groupByOperations for individual graphs
        groupByControl: [],
        description: '',
        index: []
      };
      params.name = [];
      params.groupByControl = [];
      params.cause = rca.possibleCause;
      params.description = rca.possibleCauseDescription;
      return rca.troubleshootingRecommendations?.map(
        (recommendation: TroubleshootingRecommendations) => {
          if (!recommendation.supportingGraphs) {
            return params;
          }
          return recommendation.supportingGraphs?.map((graph: GraphMetadata) => {
            params.name.push(graph.name);
            params.index.push(incrementIndex());
            // For each graph, we need to determine if we need to show groupByOperations
            params.groupByControl.push(graph.groupByControl);
            return params;
          });
        }
      );
    }
  );

  const flattenedRecommendationMetrics = nestedRecommendationMetrics?.flat(2);
  const recommendationMetricsMap = new Set(flattenedRecommendationMetrics);
  const recommendationMetrics = Array.from(recommendationMetricsMap);

  return recommendationMetrics;
};

const incrementIndex = () => {
  index = index + 1;
  return index;
};

export const getAshOperations = (metricData: any, graphPayload: any, isAdvancedView: boolean) => {
  let AshOperations: any = new Set();
  let defaultGroupBy: any = {};
  const defaultGroupByLabel = metricData.layout?.metadata?.currentGroupBy?.[0];
  let param = null;

  // In case of ADVANCED view, we show it for QUERY and DB based anomalies
  // In case of certain anomalies, we might have same graph name but we pass different parameters
  // In scenarios like above, we need to ensure we pass emptyGroupBy for certain conditions
  if (isAdvancedView) {
    const filteredPayload = graphPayload.filter(
      (payloadData: GraphQuery, payloadIdx: number) =>
        payloadIdx > 0 && payloadData.name === metricData.name
    );
    if (filteredPayload.length >= 2) {
      const filteredMetricData = filteredPayload.find(
        (payloadData: GraphQuery) =>
          payloadData.name === metricData.name &&
          metricData?.layout?.metadata?.currentGroupBy?.[0] === GraphLabel.TABLET_ID
      );
      if (filteredMetricData) {
        return Array.from(AshOperations);
      }
    } else {
      param = graphPayload.find(
        (queryParam: any, paramIdx: number) => paramIdx > 0 && queryParam.name === metricData.name
      );
    }
  } else {
    param = graphPayload.find(
      (queryParam: any, paramIdx: number) => paramIdx > 0 && queryParam.name === metricData.name
    );
  }
  metricData.layout?.metadata?.supportedGroupBy?.map((groupBy: any) => {
    if (groupBy.label === defaultGroupByLabel) {
      defaultGroupBy = groupBy;
    }

    if (!(groupBy.label === GraphLabel.QUERY_ID)) {
      AshOperations.add(groupBy.name);
    }
  });

  AshOperations = Array.from(AshOperations);

  // if (param.name === 'active_session_history' && isEmptyArray(param.groupBy)) {
  if (isEmptyArray(param?.groupBy)) {
    AshOperations.sort((a: string, b: string) => {
      return a == defaultGroupBy.name ? -1 : b == defaultGroupBy.name ? 1 : 0;
    });
  }

  return AshOperations;
};

export const getOperations = (
  metricMeasure: string,
  metricData: any,
  graphType: GraphType,
  shouldShowGroupbyOperations: boolean,
  graphPayload: any,
  isAdvancedView = false
) => {
  let uniqueOperations: any = new Set();
  let groupByOperations: any = new Set();
  const isOutlier =
    metricMeasure === MetricMeasure.OUTLIER || metricMeasure === MetricMeasure.OUTLIER_TABLES;
  if (
    metricData?.layout?.type === ASH &&
    isNonEmptyObject(metricData) &&
    isOutlier &&
    graphType === GraphType.SUPPORTING
  ) {
    groupByOperations = shouldShowGroupbyOperations
      ? getAshOperations(metricData, graphPayload, isAdvancedView)
      : Array.from(groupByOperations);
  } else {
    if (isAdvancedView) {
      groupByOperations = getAshOperations(metricData, graphPayload, isAdvancedView);
    } else {
      groupByOperations = Array.from(groupByOperations);
    }
  }

  if (isOutlier && isNonEmptyObject(metricData)) {
    metricData.data.forEach((metricItem: any) => {
      uniqueOperations.add(metricItem.name);
    });
    uniqueOperations = Array.from(uniqueOperations);
  } else if (metricData?.layout?.type === ASH && isNonEmptyObject(metricData)) {
    uniqueOperations = shouldShowGroupbyOperations
      ? getAshOperations(metricData, graphPayload, isAdvancedView)
      : Array.from(new Set());
  }
  return { uniqueOperations, groupByOperations };
};
